# PytgVoIP-Pyrogram - Pyrogram support module for Telegram VoIP Library for Python
# Copyright (C) 2020 bakatrouble <https://github.com/bakatrouble>
#
# This file is part of PytgVoIP.
#
# PytgVoIP is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PytgVoIP is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with PytgVoIP.  If not, see <http://www.gnu.org/licenses/>.
import asyncio

import pyrogram
from pyrogram import errors
from pyrogram.raw import functions, types
from pyrogram.handlers import RawUpdateHandler

from tgvoip import VoIPController, CallState, CallError, Endpoint, DataSaving, VoIPServerConfig
from tgvoip.utils import i2b, b2i, check_g


class DH:
    def __init__(self, dhc: types.messages.DhConfig):
        self.p = b2i(dhc.p)
        self.g = dhc.g
        self.resp = dhc


class VoIPCallBase:
    min_layer = 65
    max_layer = VoIPController.CONNECTION_MAX_LAYER
    is_outgoing = False

    def __init__(self, client: pyrogram.Client, use_proxy_if_available: bool = True):
        if not client.is_connected:
            raise RuntimeError('Client must be started first')
        self.client = client
        self.ctrl = VoIPController()
        self.ctrl_started = False
        self.call = None
        self.call_access_hash = None
        self.peer = None
        self.state = None
        self.dhc = None
        self.a = None
        self.g_a = None
        self.g_a_hash = None
        self.b = None
        self.g_b = None
        self.g_b_hash = None
        self.auth_key = None
        self.key_fingerprint = None
        self.call_started_handlers = []
        self.call_discarded_handlers = []
        self.call_ended_handlers = []

        if use_proxy_if_available and client.proxy:
            proxy = self.client.proxy
            self.ctrl.set_proxy(proxy['hostname'], proxy['port'], proxy['username'], proxy['password'])

        self._update_handler = RawUpdateHandler(self.process_update)
        self.client.add_handler(self._update_handler, -1)

    async def process_update(self, _, update, users, chats):
        if not isinstance(update, types.UpdatePhoneCall):
            raise pyrogram.ContinuePropagation

        call = update.phone_call
        if not self.call or not call or call.id != self.call.id:
            raise pyrogram.ContinuePropagation
        self.call = call
        if hasattr(call, 'access_hash') and call.access_hash:
            self.call_access_hash = call.access_hash

        if isinstance(call, types.PhoneCallDiscarded):
            self.call_discarded()
            raise pyrogram.StopPropagation

    def on_call_started(self, func: callable) -> callable:  # well, the conversation has started
        self.call_started_handlers.append(func)
        return func

    def on_call_discarded(self, func: callable) -> callable:  # call was discarded, not necessarily started before
        self.call_discarded_handlers.append(func)
        return func

    def on_call_ended(self, func: callable) -> callable:  # call was discarded with non-busy reason
                                                          # (was started and then discarded?)
        self.call_ended_handlers.append(func)
        return func

    def on_call_state_changed(self, func: callable) -> callable:
        if callable(func):
            self.ctrl.call_state_changed_handlers.append(lambda state: func(self, state))
        return func

    @property
    def auth_key_bytes(self) -> bytes:
        return i2b(self.auth_key) if self.auth_key is not None else b''

    @property
    def call_id(self) -> int:
        return self.call.id if self.call else 0

    def get_protocol(self) -> types.PhoneCallProtocol:
        return types.PhoneCallProtocol(min_layer=self.min_layer, max_layer=self.max_layer, udp_p2p=True,
                                       udp_reflector=True, library_versions=["2.4.4", "2.7"])

    async def get_dhc(self):
        self.dhc = DH(await self.client.send(functions.messages.GetDhConfig(version=0, random_length=256)))

    def check_g(self, g_x: int, p: int) -> None:
        try:
            check_g(g_x, p)
        except RuntimeError:
            self.call_discarded()
            raise

    def stop(self) -> None:
        async def _():
            try:
                self.client.remove_handler(self._update_handler, -1)
            except ValueError:
                pass
        asyncio.ensure_future(_())

        del self.ctrl
        self.ctrl = None

        for handler in self.call_ended_handlers:
            asyncio.iscoroutinefunction(handler) and asyncio.ensure_future(handler(self), loop=self.client.loop)

    def update_state(self, val: CallState) -> None:
        self.state = val
        self.ctrl.update_state(val)

    def call_ended(self) -> None:
        self.update_state(CallState.ENDED)
        self.stop()

    def call_failed(self, error: CallError = None) -> None:
        if error is None:
            error = self.ctrl.get_last_error() if self.ctrl and self.ctrl_started else CallError.UNKNOWN
        print('Call', self.call_id, 'failed with error', error)
        self.update_state(CallState.FAILED)
        self.stop()

    def call_discarded(self):
        # TODO: call.need_debug
        need_rate = self.ctrl and VoIPServerConfig.config.get('bad_call_rating') and self.ctrl.need_rate()
        if isinstance(self.call.reason, types.PhoneCallDiscardReasonBusy):
            self.update_state(CallState.BUSY)
            self.stop()
        else:
            self.call_ended()
        if self.call.need_rating or need_rate:
            pass  # TODO: rate

        for handler in self.call_discarded_handlers:
            asyncio.iscoroutinefunction(handler) and asyncio.ensure_future(handler(self), loop=self.client.loop)

    async def discard_call(self, reason=None):
        # TODO: rating
        if not reason:
            reason = types.PhoneCallDiscardReasonDisconnect()
        try:
            await self.client.send(functions.phone.DiscardCall(
                peer=types.InputPhoneCall(id=self.call_id, access_hash=self.call_access_hash),
                duration=self.ctrl.call_duration,
                connection_id=self.ctrl.get_preferred_relay_id(),
                reason=reason
            ))
        except (errors.CallAlreadyDeclined, errors.CallAlreadyAccepted):
            pass
        self.call_ended()

    async def _initiate_encrypted_call(self) -> None:
        config = await self.client.send(functions.help.GetConfig())  # type: types.Config
        self.ctrl.set_config(config.call_packet_timeout_ms / 1000., config.call_connect_timeout_ms / 1000.,
                             DataSaving.NEVER, self.call.id)
        self.ctrl.set_encryption_key(self.auth_key_bytes, self.is_outgoing)
        endpoints = [Endpoint(e.id, e.ip, e.ipv6, e.port, e.peer_tag) for e in self.call.connections]
        self.ctrl.set_remote_endpoints(endpoints, self.call.p2p_allowed, False, self.call.protocol.max_layer)
        self.ctrl.start()
        self.ctrl.connect()
        self.ctrl_started = True
        self.update_state(CallState.ESTABLISHED)

        for handler in self.call_started_handlers:
            asyncio.iscoroutinefunction(handler) and asyncio.ensure_future(handler(self), loop=self.client.loop)



# PytgVoIP-Pyrogram - Pyrogram support module for Telegram VoIP Library for Python
# Copyright (C) 2020 bakatrouble <https://github.com/bakatrouble>
#
# This file is part of PytgVoIP.
#
# PytgVoIP is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PytgVoIP is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with PytgVoIP.  If not, see <http://www.gnu.org/licenses/>.
import asyncio

import hashlib
from random import randint
from typing import Union

import pyrogram
from pyrogram.raw import functions, types

from tgvoip import CallState
from tgvoip.utils import i2b, b2i, calc_fingerprint
from tgvoip_pyrogram.base_call import VoIPCallBase


class VoIPOutgoingCall(VoIPCallBase):
    is_outgoing = True

    def __init__(self, user_id: Union[int, str], *args, **kwargs):
        super(VoIPOutgoingCall, self).__init__(*args, **kwargs)
        self.user_id = user_id
        self.call_accepted_handlers = []

    async def request(self):
        self.update_state(CallState.REQUESTING)
        self.peer = await self.client.resolve_peer(self.user_id)
        await self.get_dhc()
        self.a = randint(2, self.dhc.p-1)
        self.g_a = pow(self.dhc.g, self.a, self.dhc.p)
        self.g_a_hash = hashlib.sha256(i2b(self.g_a)).digest()
        self.call = (await self.client.send(functions.phone.RequestCall(
            user_id=self.peer,
            random_id=randint(0, 0x7fffffff - 1),
            g_a_hash=self.g_a_hash,
            protocol=self.get_protocol(),
        ))).phone_call
        self.update_state(CallState.WAITING)

    def on_call_accepted(self, func: callable) -> callable:  # the call was accepted by other party
        self.call_accepted_handlers.append(func)
        return func

    async def process_update(self, _, update, users, chats) -> None:
        await super(VoIPOutgoingCall, self).process_update(_, update, users, chats)
        if isinstance(self.call, types.PhoneCallAccepted) and not self.auth_key:
            await self.call_accepted()
            raise pyrogram.StopPropagation
        raise pyrogram.ContinuePropagation

    async def call_accepted(self) -> None:
        for handler in self.call_accepted_handlers:
            asyncio.iscoroutinefunction(handler) and asyncio.ensure_future(handler(self), loop=self.client.loop)

        self.update_state(CallState.EXCHANGING_KEYS)
        await self.get_dhc()
        self.g_b = b2i(self.call.g_b)
        self.check_g(self.g_b, self.dhc.p)
        self.auth_key = pow(self.g_b, self.a, self.dhc.p)
        self.key_fingerprint = calc_fingerprint(self.auth_key_bytes)
        self.call = (await self.client.send(functions.phone.ConfirmCall(
            key_fingerprint=self.key_fingerprint,
            peer=types.InputPhoneCall(id=self.call.id, access_hash=self.call_access_hash),
            g_a=i2b(self.g_a),
            protocol=self.get_protocol(),
        ))).phone_call
        await self._initiate_encrypted_call()



# PytgVoIP-Pyrogram - Pyrogram support module for Telegram VoIP Library for Python
# Copyright (C) 2020 bakatrouble <https://github.com/bakatrouble>
#
# This file is part of PytgVoIP.
#
# PytgVoIP is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PytgVoIP is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with PytgVoIP.  If not, see <http://www.gnu.org/licenses/>.

from tgvoip_pyrogram.service import VoIPService
from tgvoip_pyrogram.incoming_call import VoIPIncomingCall
from tgvoip_pyrogram.outgoing_call import VoIPOutgoingCall
from tgvoip_pyrogram.file_stream_call import VoIPFileStreamCallMixin, VoIPIncomingFileStreamCall, \
    VoIPOutgoingFileStreamCall, VoIPFileStreamService
from tgvoip_pyrogram.native_io_call import VoIPNativeIOCallMixin, VoIPIncomingNativeIOCall, \
    VoIPOutgoingNativeIOCall, VoIPNativeIOService


__all__ = ['VoIPService',
           'VoIPIncomingCall', 'VoIPOutgoingCall',
           'VoIPFileStreamService',
           'VoIPFileStreamCallMixin', 'VoIPIncomingFileStreamCall', 'VoIPOutgoingFileStreamCall',
           'VoIPNativeIOService',
           'VoIPNativeIOCallMixin', 'VoIPIncomingNativeIOCall', 'VoIPOutgoingNativeIOCall']

__version__ = '0.0.11'



# PytgVoIP-Pyrogram - Pyrogram support module for Telegram VoIP Library for Python
# Copyright (C) 2020 bakatrouble <https://github.com/bakatrouble>
#
# This file is part of PytgVoIP.
#
# PytgVoIP is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PytgVoIP is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with PytgVoIP.  If not, see <http://www.gnu.org/licenses/>.

from collections import deque
from typing import Union, List, IO

from tgvoip_pyrogram import VoIPOutgoingCall, VoIPIncomingCall, VoIPService
from tgvoip_pyrogram.base_call import VoIPCallBase


class VoIPNativeIOCallMixin(VoIPCallBase):
    def __init__(self, *args, **kwargs):
        super(VoIPNativeIOCallMixin, self).__init__(*args, **kwargs)
        self.ctrl.native_io = True

    def play(self, path: str):
        return self.ctrl.play(path)

    def play_on_hold(self, paths: List[str]):
        self.ctrl.play_on_hold(paths)

    def set_output_file(self, path: str):
        return self.ctrl.set_output_file(path)

    def clear_play_queue(self):
        self.ctrl.clear_play_queue()

    def clear_hold_queue(self):
        self.ctrl.clear_hold_queue()

    def unset_output_file(self):
        self.ctrl.unset_output_file()


class VoIPOutgoingNativeIOCall(VoIPNativeIOCallMixin, VoIPOutgoingCall):
    pass


class VoIPIncomingNativeIOCall(VoIPNativeIOCallMixin, VoIPIncomingCall):
    pass


class VoIPNativeIOService(VoIPService):
    incoming_call_class = VoIPIncomingNativeIOCall
    outgoing_call_class = VoIPOutgoingNativeIOCall



# PytgVoIP-Pyrogram - Pyrogram support module for Telegram VoIP Library for Python
# Copyright (C) 2020 bakatrouble <https://github.com/bakatrouble>
#
# This file is part of PytgVoIP.
#
# PytgVoIP is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PytgVoIP is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with PytgVoIP.  If not, see <http://www.gnu.org/licenses/>.
import asyncio
from typing import Union

import pyrogram
from pyrogram.raw import types
from pyrogram.handlers import RawUpdateHandler

from tgvoip_pyrogram.incoming_call import VoIPIncomingCall
from tgvoip_pyrogram.outgoing_call import VoIPOutgoingCall


class VoIPService:
    incoming_call_class = VoIPIncomingCall
    outgoing_call_class = VoIPOutgoingCall

    def __init__(self, client: pyrogram.Client, receive_calls=True):
        self.client = client
        self.incoming_call_handlers = []
        if receive_calls:
            client.add_handler(RawUpdateHandler(self.update_handler), -1)
        client.on_message()

    def get_incoming_call_class(self):
        return self.incoming_call_class

    def get_outgoing_call_class(self):
        return self.outgoing_call_class

    def on_incoming_call(self, func) -> callable:
        self.incoming_call_handlers.append(func)
        return func

    async def start_call(self, user_id: Union[str, int]):
        call = self.get_outgoing_call_class()(user_id, client=self.client)
        await call.request()
        return call

    def update_handler(self, _, update, users, chats):
        if isinstance(update, types.UpdatePhoneCall):
            call = update.phone_call
            if isinstance(call, types.PhoneCallRequested):
                async def _():
                    voip_call = self.get_incoming_call_class()(call, client=self.client)
                    for handler in self.incoming_call_handlers:
                        asyncio.iscoroutinefunction(handler) and asyncio.ensure_future(handler(voip_call),
                                                                                       loop=self.client.loop)
                asyncio.ensure_future(_(), loop=self.client.loop)
        raise pyrogram.ContinuePropagation



# PytgVoIP-Pyrogram - Pyrogram support module for Telegram VoIP Library for Python
# Copyright (C) 2020 bakatrouble <https://github.com/bakatrouble>
#
# This file is part of PytgVoIP.
#
# PytgVoIP is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PytgVoIP is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with PytgVoIP.  If not, see <http://www.gnu.org/licenses/>.
import asyncio

import hashlib
from random import randint

import pyrogram
from pyrogram import errors
from pyrogram.raw import types, functions

from tgvoip import CallState
from tgvoip.utils import i2b, b2i, calc_fingerprint
from tgvoip_pyrogram.base_call import VoIPCallBase


class VoIPIncomingCall(VoIPCallBase):
    def __init__(self, call: types.PhoneCallRequested, *args, **kwargs):
        super(VoIPIncomingCall, self).__init__(*args, **kwargs)
        self.call_accepted_handlers = []
        self.update_state(CallState.WAITING_INCOMING)
        self.call = call
        self.call_access_hash = call.access_hash

    async def process_update(self, _, update, users, chats):
        await super(VoIPIncomingCall, self).process_update(_, update, users, chats)
        if isinstance(self.call, types.PhoneCall) and not self.auth_key:
            await self.call_accepted()
            raise pyrogram.StopPropagation
        raise pyrogram.ContinuePropagation

    def on_call_accepted(self, func: callable) -> callable:  # telegram acknowledged that you've accepted the call
        self.call_accepted_handlers.append(func)
        return func

    async def accept(self) -> bool:
        self.update_state(CallState.EXCHANGING_KEYS)
        if not self.call:
            self.call_failed()
            raise RuntimeError('call is not set')
        await self.get_dhc()
        self.b = randint(2, self.dhc.p-1)
        self.g_b = pow(self.dhc.g, self.b, self.dhc.p)
        self.g_a_hash = self.call.g_a_hash
        try:
            self.call = (await self.client.send(functions.phone.AcceptCall(
                peer=types.InputPhoneCall(id=self.call_id, access_hash=self.call_access_hash),
                g_b=i2b(self.g_b),
                protocol=self.get_protocol()
            ))).phone_call
        except errors.Error as e:
            if e.ID == 'CALL_ALREADY_ACCEPTED':
                self.stop()
                return True
            elif e.ID == 'CALL_ALREADY_DECLINED':
                self.call_discarded()
                return False
            raise e
        if isinstance(self.call, types.PhoneCallDiscarded):
            print('Call is already discarded')
            self.call_discarded()
            return False
        return True

    async def call_accepted(self) -> None:
        for handler in self.call_accepted_handlers:
            asyncio.iscoroutinefunction(handler) and asyncio.ensure_future(handler(self), loop=self.client.loop)

        if not self.call.g_a_or_b:
            print('g_a is null')
            self.call_failed()
            return
        if self.g_a_hash != hashlib.sha256(self.call.g_a_or_b).digest():
            print('g_a_hash doesn\'t match')
            self.call_failed()
            return
        self.g_a = b2i(self.call.g_a_or_b)
        self.check_g(self.g_a, self.dhc.p)
        self.auth_key = pow(self.g_a, self.b, self.dhc.p)
        self.key_fingerprint = calc_fingerprint(self.auth_key_bytes)
        if self.key_fingerprint != self.call.key_fingerprint:
            print('fingerprints don\'t match')
            self.call_failed()
            return
        await self._initiate_encrypted_call()



# PytgVoIP-Pyrogram - Pyrogram support module for Telegram VoIP Library for Python
# Copyright (C) 2020 bakatrouble <https://github.com/bakatrouble>
#
# This file is part of PytgVoIP.
#
# PytgVoIP is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PytgVoIP is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with PytgVoIP.  If not, see <http://www.gnu.org/licenses/>.

from collections import deque
from typing import Union, List, IO

from tgvoip_pyrogram import VoIPOutgoingCall, VoIPIncomingCall, VoIPService
from tgvoip_pyrogram.base_call import VoIPCallBase


class VoIPFileStreamCallMixin(VoIPCallBase):
    def __init__(self, *args, **kwargs):
        super(VoIPFileStreamCallMixin, self).__init__(*args, **kwargs)
        self.input_files = deque()
        self.hold_files = deque()
        self.output_file = None
        self.ctrl.set_send_audio_frame_callback(self._read_frame)
        self.ctrl.set_recv_audio_frame_callback(self._write_frame)

    def __del__(self):
        self.clear_play_queue()
        self.clear_hold_queue()
        self.unset_output_file()

    def play(self, f: Union[IO, str]):
        if isinstance(f, str):
            f = open(f, 'rb')
        elif not hasattr(f, 'mode') or 'b' not in f.mode or not any(m in f.mode for m in 'rxa+'):
            print('file must be opened in binary reading/updating mode')
            return
        self.input_files.append(f)

    def play_on_hold(self, files: List[Union[IO, str]]):
        if not isinstance(files, (list, tuple, set)):
            print('list, tuple or set expected, got {}'.format(type(files)))
            return
        self.clear_hold_queue()
        for f in files:
            if isinstance(f, str):
                f = open(f, 'rb')
            elif not hasattr(f, 'mode') or 'b' not in f.mode or not any(m in f.mode for m in 'rxa+'):
                print('file must be opened in binary reading/updating mode')
                continue
            self.hold_files.append(f)

    def set_output_file(self, f: Union[IO, str]):
        if isinstance(f, str):
            f = open(f, 'wb')
        elif 'b' not in f.mode or not any(m in f.mode for m in 'wxa+'):
            print('file must be opened in binary writing/updating mode')
            return
        self.output_file = f

    def clear_play_queue(self):
        for f in self.input_files:
            f.close()
        self.input_files.clear()

    def clear_hold_queue(self):
        for f in self.hold_files:
            f.close()
        self.hold_files.clear()

    def unset_output_file(self):
        if self.output_file:
            self.output_file.close()
        self.output_file = None

    def _read_frame(self, length: int) -> bytes:
        frame = b''
        if len(self.input_files):
            frame = self.input_files[0].read(length)
            if len(frame) != length:
                self.input_files[0].close()
                self.input_files.popleft()
        elif len(self.hold_files):
            frame = self.hold_files[0].read(length)
            if len(frame) != length:
                self.hold_files[0].seek(0)
                self.hold_files.rotate(-1)
        return frame

    def _write_frame(self, frame: bytes) -> None:
        if self.output_file:
            self.output_file.write(frame)


class VoIPOutgoingFileStreamCall(VoIPFileStreamCallMixin, VoIPOutgoingCall):
    pass


class VoIPIncomingFileStreamCall(VoIPFileStreamCallMixin, VoIPIncomingCall):
    pass


class VoIPFileStreamService(VoIPService):
    incoming_call_class = VoIPIncomingFileStreamCall
    outgoing_call_class = VoIPOutgoingFileStreamCall



#!/usr/bin/env python3

# PytgVoIP-Pyrogram - Pyrogram support module for Telegram VoIP Library for Python
# Copyright (C) 2020 bakatrouble <https://github.com/bakatrouble>
#
# This file is part of PytgVoIP.
#
# PytgVoIP is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PytgVoIP is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with PytgVoIP.  If not, see <http://www.gnu.org/licenses/>.

import os
import re

from setuptools import setup


def get_version():
    with open('tgvoip_pyrogram/__init__.py', encoding='utf-8') as f:
        version = re.findall(r"__version__ = '(.+)'", f.read())[0]
        if os.environ.get('DEVELOP') is not None:
            version += '+develop'
        return version


def get_long_description():
    with open(os.path.join(os.path.abspath(os.path.dirname(__file__)), 'README.md'), encoding='utf-8') as f:
        return f.read()


setup(
    name='pytgvoip_pyrogram',
    version=get_version(),
    license='LGPLv3+',
    author='bakatrouble',
    author_email='bakatrouble@gmail.com',
    description='Pyrogram support module for Telegram VoIP Library for Python',
    long_description=get_long_description(),
    long_description_content_type='text/markdown',
    url='https://github.com/bakatrouble/pytgvoip_pyrogram',
    keywords='telegram messenger voip library python pyrogram',
    project_urls={
        'Tracker': 'https://github.com/bakatrouble/pytgvoip_pyrogram/issues',
        'Community': 'https:/t.me/pytgvoip',
        'Source': 'https://github.com/bakatrouble/pytgvoip_pyrogram',
    },
    python_required='~=3.5',
    packages=['tgvoip_pyrogram'],
    install_requires=[
        'pytgvoip >= 0.0.7',
        'pyrogram >= 1.0.0'
    ],
    zip_safe=False,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Internet',
        'Topic :: Communications',
        'Topic :: Communications :: Internet Phone',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)
